from .. import base
from .. import item
from .. import exp
from ... import file
import json
class PrevStrDeal(base.BaseDeal):
    def has_prev(self):
        return 1
    def has_deal(self):
        return 0
    def init(self, left = '"', right= '"', single_line = False, note = False, translate = False):
        self.left = left
        self.right = right
        self.ll = len(left)
        self.lr = len(right)
        self.single_line = single_line
        self.note = note
        self.translate = translate
    def json_loads(self, s):
        x = s
        cd = None
        if type(x)==bytes:
            x, cd = file.decode_c(x)
        rs = json.loads(x)
        if type(s)==bytes:
            rs = rs.encode(cd)
        return rs
    def do_translate(self, s):
        """
            取巧直接调用json
        """
        qt = self.like('"',s)
        ql = self.like("\\", s)
        et = self.like("\n", s)
        tr = self.like("\r", s)
        nt = self.like("",s)
        pt = ql+qt
        arr = s.split(pt)
        arr = [k.replace(qt, pt) for k in arr]
        s = pt.join(arr)
        #s = s.replace(qt, ql+qt)
        s = s.replace(tr, nt)
        arr = s.split(et)
        outs = [self.json_loads(qt+k+qt) for k in arr]
        outs = et.join(outs)
        return outs
    def prev(self, buffer, queue, pos):
        cl = buffer.read(self.ll)
        if not self.same(self.left, cl):
            return False
        buffer.pop_read(self.ll)
        rm = buffer.full()
        rm_pos = pos.get()
        if len(rm.strip())>0 and not self.note:
            print("left:", self.left, rm)
            raise exp.FormatExp("unexcept char before string", pos.get(), rm)
        pos.update(rm)
        pos.update(cl)
        buffer.clean()
        tmp = cl[:0]
        ctmp = tmp[:0]
        while True:
            if self.same(self.right, ctmp[-self.lr:]):
                break
            c = buffer.read(1,1)
            if len(c)==0:
                pos.update(tmp)
                raise exp.FormatExp("unexcept file end while reading string", pos.get())
            tmp += c
            if self.same("\\", c):
                c = buffer.read(1,1)
                if len(c)==0:
                    pos.update(tmp)
                    raise exp.FormatExp("unexcept file end while reading string", pos.get())
                tmp += c
                continue
            ctmp+=c
        buffer.clean()
        xtmp = tmp[:-self.lr]
        if not self.note and self.single_line and xtmp.find(self.like("\n", xtmp))>=0:
            print("left:",self.left, "right:", self.right)
            raise exp.FormatExp("contain enter in single line string", pos.get(), tmp)
        if self.translate:
            xtmp = self.do_translate(xtmp)
        curr_pos = pos.get()
        pos.update(tmp)
        if self.note:
            if len(rm)>0:
                queue.append(item.PrevItem(rm.rstrip(), rm_pos, is_val = 1, src='str'))
            return True
        queue.append(item.PrevItem(xtmp, curr_pos, self.id(), is_val = 1, src='str'))
        return True

pass



#

from .. import xf
from .. import ioc
from ..base import Base
from ..ioc import wrap

class DeepFc(Base):
    def init(self, fcs, default = None):
        self.default = default
        self.fc = None
        self.next=None
        if len(fcs)>0:
            self.fc = fcs[0]
            self.next = DeepFc(fcs[1:], default)
    def call(self, data):
        if self.fc is None:
            return self.default
        return self.fc(data, self.next)

pass
@wrap.obj(id = "buildz.auto.deal.fill")
@wrap.obj_args("ioc, confs")
class Fill(Base):
    def init(self, mg):
        self.mg = mg
    def call(self, orders, default=None):
        if type(orders)==str:
            orders = xf.loads(orders)
        if type(orders)==str:
            orders = [orders]
        fcs = [self.mg.get(id) for id in orders]
        return DeepFc(fcs,default)

pass


try:
    import psycopg
except ModuleNotFoundError:
    try:
        import psycopg2 as psycopg
    except ModuleNotFoundError:
        raise Exception("module not found, try: pip install psycopg")
import sys
from .basez import SimpleDv, fetch
from .structz import CMD
class Db(SimpleDv):
    # func to impl:
    def to_list(self, rst):
        rows = self.cursor.description
        keys = [k.name.lower() for k in rows]
        result = []
        result.append(keys)
        if rst is None or len(rst)==0:
            return result
        result += rst
        return result
    def new_con(self):
        return psycopg.connect(host=self.host, 
            port = self.port, user =self.user, 
            password =self.pwd, dbname = self.db)
    def new_cursor(self):
        return self.con.cursor()
    def init(self, *argv, **maps):
        if self.port is None:
            self.port = 5432
        if self.user is None:
            self.user = "postgres"
        pass

pass
def build(argv, conf):
    dv = Db(*fetch(argv))
    return dv
def buildbk(argv, conf):
    return CMD(make(argv, conf))

pass



#

def build(dv,*a,**b):
    if dv == 'mysql':
        from .mysqlz import build as _build
    elif dv == 'oracle':
        from .oraclez import build as _build
    elif dv == 'clickhouse':
        from .clickhousez import build as _build
    elif dv == "postgresql":
        from .postgresqlz import build as _build
    else:
        raise Exception(f"not impl dv: {dv}")
    return _build(*a, **b)

pass
    




try:
    import pymysql
except ModuleNotFoundError:
    raise Exception("module not found, try: pip install pymysql")
import sys
from .basez import SimpleDv, fetch
from .structz import CMD
class Db(SimpleDv):
    # func to impl:
    def to_list(self, rst):
        result = []
        if rst is None or len(rst)==0:
            return result
        a = rst[0]
        keys = list(a.keys())
        result.append(keys)
        for obj in rst:
            v = [obj[k] for k in keys]
            result.append(v)
        return result
    def new_con(self):
        return pymysql.connect(host=self.host, 
            port = self.port, user =self.user, 
            password =self.pwd, database = self.db,
            charset='utf8',init_command="SET SESSION time_zone='+08:00'")
    def new_cursor(self):
        return self.con.cursor(pymysql.cursors.DictCursor)
    def init(self, *argv, **maps):
        if self.port is None:
            self.port = 3306
        pass

pass
def build(argv, conf):
    dv = Db(*fetch(argv))
    return dv
def buildbk(argv, conf):
    return CMD(make(argv, conf))

pass




from .. import base
from .. import itemz
import re
import json
class ValDeal(base.BaseDeal):
    def deal(self, obj, conf):
        if not obj.check(is_val=1):
            return None
        val = obj.val
        if type(val) in [list, dict, tuple]:
            return None
        val = conf.s(json.dumps(val, ensure_ascii=False))
        return itemz.ShowItem(val, obj.deep, is_val=1, is_done=1)

pass 



            

                


#
from ..ioc.base import Base, EncapeData
from buildz import xf
class ValDeal(Base):
    """
        数据val:
            {
                // 查找id，可选
                id: id
                type: val
                val|data: 任何数据
            }
        简写:
            [[id, val], data]
        极简:
            [val, data]

        例子:
            [val, 100元] // 字符串"100元"
        注: val是默认数据项的类型，如果data不是list或map，可以直接只写data：
            100元
    """
    def deal(self, edata:EncapeData):
        data = edata.data
        if type(data)==list:
            return data[-1]
        return xf.get_first(data, "val", "data")

pass



#
from ..ioc.base import Base, EncapeData
from .base import FormatData,FormatDeal
from buildz import xf, pyz
import os
dp = os.path.dirname(__file__)
join = os.path.join
class ObjectVarDeal(FormatDeal):
    """
        对象变量ovar:
            {
                id:id
                type: ovar
                source: string
                ovar|key: string
                info: null
            }
        简写:
            [[id, ovar], source, key, info]
            [ovar, source, key]
        例:
            [ovar, obj.test, id] //返回对象id=obj.test的变量id
    """
    def init(self, fp_lists = None, fp_defaults = None):
        self.singles = {}
        self.sources = {}
        super().init("ObjectVarDeal", fp_lists, fp_defaults, 
            join(dp, "conf", "ovar_lists.js"))
    def deal(self, edata:EncapeData):
        sid = edata.sid
        data = edata.data
        conf = edata.conf
        data = self.format(data)
        src = edata.src
        source = xf.g1(data, source=None, src=None)
        #key = xf.g(data, key=0)
        key = xf.get_first(data, "ovar", "key", "data")
        info = xf.g(data, info=None)
        if info is not None:
            info = self.get_obj(info, src = edata.src, info = edata.info)
        else:
            info = edata.info
        if source is not None:
            source = conf.get_obj(source, info = info)
        if source is None:
            source = src
        if source is None:
            raise Exception(f"not object for key {key}")
        key = getattr(source, key)
        return key

pass




class BaseDeal:
    def fmt(self, s, num, spc = " "):
        spc = self.like(spc, s)
        spc = spc*num
        et = self.like("\n", s)
        arr = s.split(et)
        arr = [spc+k for k in arr]
        rs = et.join(arr)
        return rs
    def __call__(self, queue, conf):
        return self.deal(queue, conf)
    def deal(self, queue, conf):
        "impl"
        return None
    def sp(self):
        return super(self.__class__, self)
    def regist(self, mgs):
        if self._id is None:
            self._id = mgs.regist()
    def id(self):
        return self._id
    def same(self, s, target):
        return self.like(s, target) == target
    def like(self, s, target):
        if type(target) ==type(s):
            return s
        if type(s)==str:
            return s.encode()
        return s.decode()
    def __str__(self):
        return "BaseDeal"
    def __repr__(self):
        return str(self)
    def __init__(self, *argv, **maps):
        self._id = None
        self.init(*argv, **maps)
    def init(self, *argv, **maps):
        pass


#coding=utf-8
from buildz import xf, pyz
from buildz.xf import g as xg
from buildz.base import Base as BzBase
import json
import builtins
typez = builtins.type
class IOCError(Exception):
    def __init__(self, msg):
        super().__init__(msg)

pass
class IdNotFoundError(IOCError):
    pass
class Base(BzBase):
    def update_maps(self, maps, src, replace=1):
        xf.deep_update(maps, src, replace)
    def __init__(self, *args, **maps):
        self.init(*args, **maps)
    def init(self, *args, **maps):
        pass
    def __call__(self, *args, **maps):
        return self.deal(*args, **maps)
    def deal(self, *args, **maps):
        raise Exception("unexcept touch")
        return None

pass
class EncapeData(Base):
    def str(self):
        return f"EncapeData(data={self.data}, type = {self.type})"
    """
        包含data id对应的配置，配置文件id，配置文件对象
        [object.test, call, ]
    """
    def __init__(self, data, conf=None, local = False, type = None, src = None, info = None, confs = None):
        """
            data: 配置数据
            conf: 配置数据对应的配置文件的管理器
            local: 是否是locals的数据（配置文件局部数据）
            type: 配置数据的type字段
            src: 源对象，配置数据生成的对象调用conf获取对象，会有这个字段，目前只有object会放这个字段，其他要么透传要么不传
            info: 额外的调用信息，目前只有object会用到里面的id字段，作为单例额外输入
        """
        if typez(data)==dict:
            pid = xf.g1(data, parent=None, template=None, temp=None)
            if pid is not None:
                data = dict(data)
                for pkey in "parent,temp,template".split(","):
                    if pkey in data:
                        del data[pkey]
                pids = pid
                if typez(pids)!=list:
                    pids = [pid]
                for pid in pids:
                    pedt = conf.get_data(pid, local=True, search_confs = True,src = src, info = info)
                    if pedt is None:
                        raise IOCError("unfind parend: "+pid)
                    pdt = pedt.data
                    if typez(pdt)!=dict:
                        raise IOCError("only dict can be a parent: "+pid)
                    self.update_maps(data, pdt, replace=0)
        self.data = data
        if conf is not None:
            self.sid = conf.id
            if type is None:
                type = conf.confs.get_data_type(data, 0, conf.default_type())
            if confs is None:
                confs = conf.confs
        else:
            self.sid = None
        self.src = src
        self.conf = conf
        self.confs = confs
        self.local = local
        self.type = type
        self.info = info
    def deal(self, remove = False):
        if self.conf is None:
            return self.confs.get(self, src = self.src, info=self.info, remove = remove)
        return self.conf.get(self, src = self.src, info=self.info, remove = remove)


pass



#coding=utf-8
from . import xf
class Cmd:
    def close(self):
        self.running = False
    def __init__(self, obj):
        self.running = False
        self.obj = obj
        self.tmp = None
        self.exits = "exit,close,quit".split(",")
    def __call__(self):
        self.run()
    def doc(self, *arr):
        if len(arr)>0:
            rfc = getattr(self.obj, arr[0])
        else:
            rfc = self.obj
        return rfc.__doc__
    def help(self, *arr):
        if len(arr)>0:
            rfc = getattr(self.obj, arr[0])
        else:
            rfc = self.obj
        help(rfc)
    def dir(self, *arr):
        if len(arr)>0:
            rfc = getattr(self.obj, arr[0])
        else:
            rfc = self.obj
        arr = rfc.__dir__()
        arr = [k for k in arr if k[:2]!="__"]
        return arr
    def dict(self, *arr):
        if len(arr)>0:
            rfc = getattr(self.obj, arr[0])
        else:
            rfc = self.obj
        return rfc.__dict__
    def get(self, *arr):
        if len(arr)>0:
            return getattr(self.obj, arr[0])
        return self.obj
    def set(self, key, val):
        setattr(self.obj,key, val)
        return getattr(self.obj, key)
    def run(self):
        self.running = True
        while self.running:
            s = input(":").strip()
            if s.strip()=="":
                continue
            if s in self.exits:
                break
            try:
                arr = xf.loads(s)
                if type(arr)!=list:
                    if type(arr)==dict:
                        rst = []
                        for k,v in arr.items():
                            rst+=[k,v]
                        arr = rst
                    else:
                        arr = [arr]
                fc = arr[0]
                if not hasattr(self.obj, fc):
                    fc = fc.split(".")[-1]
                    fc = getattr(self, fc)
                else:
                    fc = getattr(self.obj, fc)
                params = arr[1:]
                tmp = fc(*params)
                print(tmp)
            except Exception as exp:
                import traceback
                print(f"[CMD] error:{exp}")
                traceback.print_exc()

pass




import sys
import os
import time
from buildz import xf
FP = "run.conf"
def find(arr, k, base = 0):
    for i in range(base, len(arr)):
        if arr[i]==k:
            return i
    return -1

pass
if __name__ == "__main__":
    from dv import build
    from dv.structz import CMD
else:
    from .dv import build
    from .dv.structz import CMD
def test(fp):
    obj = xf.loads(xf.fread(fp))
    dv = obj['dv']
    db_url = obj['db']
    db_url = obj[db_url]
    user = obj['user']
    pwd = obj['pwd']
    src = obj['src']
    out = obj['out']
    sqls = xf.fread(src)
    sqls = sqls.replace("\r\n", "\n").rstrip().split("\n")
    sqls_strip = [k.strip() for k in sqls]
    i = find(sqls_strip, "!!begin")
    bi = i
    if i < 0:
        bi=0
    j = find(sqls_strip, "!!end", bi)
    sqls = sqls[i+1:j]
    sqls = [k for k in sqls if k.lstrip()[:2]!="--"]
    sqls = "\n".join(sqls)
    sqls = sqls.split(";")
    print(f"[TESTZ] sqls:{sqls}")
    cmd = CMD(build(dv, [db_url, user, pwd], obj))
    cmd.dv.begin()
    print("[TESTZ] A")
    with open(out, 'wb') as f:
        for sql in sqls:
            if sql.strip()=="":
                continue
            if sql.strip() == "exit":
                break
            print("[TESTZ] B")
            rst = cmd.single(sql)
            print("[TESTZ] C")
            f.write(rst.encode("utf-8"))
    cmd.close()

pass
class Runner:
    def __init__(self, fp):
        self.fp = fp
    def init(self):
        self.obj = xf.loads(xf.fread(self.fp))
        self.fps = [self.fp, self.obj['src']]
        self.fps = [os.path.abspath(fp) for fp in self.fps]
        self.curr = [0,0]
    def run(self):
        while True:
            secs = [os.path.getmtime(fp) for fp in self.fps]
            if self.curr != secs:
                print(time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()), "done update")
                self.init()
                self.curr = secs
                test(self.fp)
            time.sleep(float(self.obj['sec']))

pass

def run(fp):
    r = Runner(fp)
    r.init()
    r.run()

pass
def main():
    fp = FP
    if len(sys.argv)>1:
        fp = sys.argv[1]
    return run(fp)

pass

if __name__=="__main__":
    main()

pass



#coding=utf-8
from ..ioc.base import Base, EncapeData
from .base import FormatData,FormatDeal
from buildz import xf, pyz
import os
dp = os.path.dirname(__file__)
join = os.path.join
class IOCObjectDeal(FormatDeal):
    """
        ioc字段ioc:
            {
                id:id
                type: ioc
                //default conf
                key: string = conf, confs, sid 
            }
        简写:
            [[id, ioc], key]
            [ioc]
        例:
            [ioc, conf] //返回ioc内部数据的conf字段
    """
    def init(self, fp_lists=None, fp_defaults=None):
        super().init("IOCObjectDeal", fp_lists, fp_defaults, join(dp, "conf", "ioc_lists.js"), None)
    def deal(self, edata:EncapeData):
        data = edata.data
        data = self.fill(data)
        key = xf.get_first(data, 'ioc', 'key')
        if not hasattr(edata, key):
            return None
        return getattr(edata, key)

pass



#
from ..ioc.base import Base, EncapeData
from .base import FormatData,FormatDeal
from buildz import xf, pyz
import os
dp = os.path.dirname(__file__)
join = os.path.join
class CallsDeal(FormatDeal):
    """
    函数调用序列calls:
        {
            id:id
            type:calls
            calls: [
                item_conf,
                ...
            ]
        }
    简写:
        [[id, calls], calls]
        [calls, calls]
    例:
        //顺序调用buildz.pyz.pyexe()和buildz.pyz.pypkg()
        [[main_call, calls], [[call, buildz.pyz.pyexe], [call, buildz.pyz.pypkg]]] 
    """
    def init(self, fp_lists = None, fp_defaults = None):
        super().init("CallsDeal", fp_lists, fp_defaults, 
            join(dp, "conf", "calls_lists.js"),
            join(dp, "conf", "calls_defaults.js"))
    def deal(self, edata:EncapeData):
        sid = edata.sid
        data = edata.data
        conf = edata.conf
        data = self.format(data)
        src = edata.src
        info = edata.info
        calls = xf.g1(data, calls=[], data = [])
        rst = None
        for call in calls:
            rst = self.get_obj(call, conf, src, info=info)
        return rst

pass



#coding=utf-8
from ..ioc.base import Base, EncapeData
from .base import FormatData,FormatDeal
from buildz import xf, pyz
import os
dp = os.path.dirname(__file__)
join = os.path.join
class MapDeal(FormatDeal):
    """
    map:
        {
            id:id
            type:map
            map|data: {
                key1: item_conf,
                ...
            }
        }
    简写:
        [[id, map], data]
        [map, data]
    例:
        [map, {obj:[ref, obj.test], path: [env, path]}] // 返回{obj:对对象obj.test的索引, path: 环境变量path的值}
    """
    def init(self, fp_lists=None, fp_defaults=None):
        super().init("MapDeal", fp_lists, fp_defaults, join(dp, "conf", "map_lists.js"), None)
    def deal(self, edata:EncapeData):
        data = edata.data
        conf = edata.conf
        data = self.fill(data)
        maps = xf.g1(data, map = {}, data={})
        rst = {k:self.get_obj(maps[k], conf, edata.src, edata.info) for k in maps}
        return rst

pass



from .. import base
from .. import item
from .. import exp
from ... import file
import json
from . import lr
from ... import code as codez
def init():
    cs = "abfnrtv\\'\"?0123456789xcde"
    maps = {k:k.encode()[0] for k in cs}
    global c2b
    c2b = maps
    global symbal_a
    global a_map
    a_map = [0]*256
    s  = "abfnrtv\\'\"?"
    ts = b"\a\b\f\n\r\t\v\\\'\"\?"
    for c,t in zip(s, ts):
        a_map[c2b[c]] = t
    symbal_a = [False]*256
    for c in s:
        symbal_a[c2b[c]] = True
    global id_0,id_a,id_x
    id_0 = b'0'[0]
    id_a = b'a'[0]
    id_x = b'x'[0]

pass

init()
def translate_bts(bts):
    i = 0
    rs = []
    while i<len(bts):
        c = bts[i]
        i+=1
        if c!=c2b['\\']:
            rs.append(c)
            continue
        x0 = bts[i]
        i+=1
        if symbal_a[x0]:
            rs.append(a_map[x0])
            continue
        v = x0-id_0
        if v>=0 and v<=7:
            tmp = v
            for j in range(2):
                if i+j+1>=len(bts):
                    break
                xi = bts[i+j+1]
                vi = xi-id_0
                if vi>=0 and vi<=7:
                    i+=1
                    tmp = (rs<<3)+vi
                else:
                    break
            rs.append(tmp)
            continue
        if x0 == id_x:
            tmp = 0
            for j in range(2):
                if i+j+1>=len(bts):
                    raise Exception("\\xXX error")
                xi = bts[i+j+1]
                vi = xi-id_0
                if vi<0 or vi>7:
                    vi = xi-id_a
                    if vi<0 or vi > 5:
                        raise Exception("\\xXX error")
                    vi +=10
                tmp=(tmp<<4)+vi
            rs.append(tmp)
            continue
        rs.append(c)
        rs.append(x0)
        #i-=1
    return bytes(rs)

pass

class PrevStrDeal(lr.LRDeal):
    def types(self):
        if not self.deal_build:
            return []
        return ['str']
    def build(self, obj):
        if type(obj.val)==list:
            return None
        if obj.is_val:
            print("error str build")
            return obj
        obj.is_val = 1
        if self.translate:
            print("error str build tr")
            val = obj.val
            val = self.do_translate(val)
            obj.val = val
        return obj
    def prepare(self, mg):
        super().prepare(mg)
        self.label_l2 = mg.like("\\")
        self.label_qt = mg.like('"')
        self.label_et = mg.like("\n")
        self.label_lr = mg.like("\r")
        self.label_nl = mg.like("")
        self.et_in_right = self.right.count(self.label_et)
    def init(self, left = '"', right= '"', single_line = False, note = False, translate = False, deal_build = False):
        super().init(left, right, 'str')
        self.single_line = single_line
        self.note = note
        self.translate = translate
        self.deal_build = deal_build
    def json_loads(self, s):
        x = s
        cd = None
        if type(x)==bytes:
            x, cd = file.decode_c(x)
        rs = json.loads(x)
        if type(s)==bytes:
            rs = rs.encode(cd)
        return rs
    def do_translate(self, s):
        is_bytes = type(s)==bytes
        if not is_bytes:
            s = s.encode("utf-8")
        #s = translate_bts(s)
        s = codez.ubytes(s, "utf-8")
        if not is_bytes:
            s = s.decode("utf-8")
        return s
        """
            取巧直接调用json
        """
        qt = self.label_qt
        ql = self.label_l2
        et = self.label_et
        tr = self.label_lr
        nt = self.label_nl
        pt = ql+qt
        arr = s.split(pt)
        arr = [k.replace(qt, pt) for k in arr]
        s = pt.join(arr)
        #s = s.replace(qt, ql+qt)
        s = s.replace(tr, nt)
        arr = s.split(et)
        outs = [self.json_loads(qt+k+qt) for k in arr]
        outs = et.join(outs)
        return outs
    def deal(self, buffer, rst, mg):
        cl = buffer.read(self.ll)
        if cl != self.left:
            return False
        rm = buffer.full().strip()
        buffer.clean2read(self.ll)
        if len(rm)>0:
            if not self.note:
                raise Exception(f"unexcept char before string: {rm}")
            else:
                rst.append(item.Item(rm, type = "str", is_val = 0))
        tmp = cl[:0]
        ctmp = tmp[:0]
        do_judge = 1
        mark_et = 0
        mark_l2 = 0
        while True:
            if do_judge and self.right == buffer.rget(self.lr):
                break
            c = buffer.read_cache(1)
            if do_judge and c == self.label_et:
                mark_et += 1
            if len(c)==0:
                if self.single_line and self.note:
                    break
                raise Exception(f"unexcept string end while reading str")
            do_judge = 1
            if c == self.label_l2:
                mark_l2 = 1
                do_judge = 0
                c = buffer.read_cache(1)
                if len(c)==0:
                    raise Exception(f"unexcept string end while reading str")
        data = buffer.full()
        data = data[:-self.lr]
        buffer.clean()
        mark_et -= self.et_in_right
        if self.single_line and mark_et>0:
            print("left:",self.left, "right:", self.right)
            raise Exception(f"contain enter in single line string")
        if self.translate and mark_l2:
            data = self.do_translate(data)
        if self.note:
            return True
        rst.append(item.Item(data, type='str', is_val = 1))
        return True

pass



#import pymysql
import sys
class ItDv:
    def begin(self):
        pass
    def close(self):
        pass
    def is_open(self):
        return False
    def commit(self):
        pass
    def refresh(self):
        pass
    def check_query(self, s):
        return False
    def query(self, sql, vals=()):
        # return list, first row is key
        return []
    def execute(self, sql, vals=()):
        return None
    def sql_tables(self):
        return ""

pass

class CMD:
    def begin(self):
        return self.dv.begin()
    def close(self):
        return self.dv.close()
    def __init__(self, dv, simple_format = True):
        self.s_rst = ""
        self.dv = dv
        self.simple_format = simple_format
    def __enter__(self, *argv, **maps):
        self.dv.begin()
        return self
    def __exit__(self, *argv, **maps):
        self.dv.close()
    def exec(self, fc, sql, vals = ()):
        need_close = False
        if not self.dv.is_open():
            self.dv.begin()
            need_close = True
        rst = fc(sql, vals)
        if need_close:
            self.dv.close()
        return rst
    def query(self, sql, vals = ()):
        return self.exec(self.dv.query, sql, vals)
    def execute(self, sql, vals = ()):
        return self.exec(self.dv.execute, sql, vals)
    def s_print(self, *args):
        args = [str(k) for k in args]
        s = " ".join(args)
        s = s+"\n"
        self.s_rst += s
    def s_flush(self):
        out = self.s_rst
        self.s_rst = ""
        return out
    def rp(self, s):
        return s
        rps = ["\n\\n","\r\\r","\t\\t"]
        for rp in rps:
            s = s.replace(rp[0], rp[1:])
        return s
    def sz(self, s):
        try:
            s = s.encode("gbk")
        except Exception as exp:
            print("SZ exp:", exp)
        return min(100, len(s))
    def jstr(self, obj):
        import datetime
        import decimal
        if type(obj) == datetime.datetime:
            obj = str(obj) 
        if type(obj) == decimal.Decimal:
            obj = str(obj) 
        if type(obj) == bytes:
            obj = list(obj)[0]
        import json
        rs = json.dumps(obj, ensure_ascii=0)
        return rs
    def tr_sz(self, s, sz):
        try:
            s = s.encode("gbk")
            s = s+(b" "*(sz-len(s)))
            s = s.decode("gbk")
        except Exception as exp:
            print("TR_SZ exp:", exp)
        return s
    def format(self, arr):
        arr = [[self.rp(k) for k in obj] for obj in arr]
        if self.simple_format and len(arr)>0:
            sz = [[self.sz(k) for k in obj] for obj in arr]
            l = len(arr[0])
            szs = [max([obj[i] for obj in sz]) for i in range(l)]
            arr = [[self.tr_sz(obj[i], szs[i]) for i in range(l)] for obj in arr]
        arr = [" | ".join(k) for k in arr]
        arr = ["[[ "+k+" ]]" for k in arr]
        return "\n".join(arr)
    def single(self, s):
        s=s.strip()
        if s == "commits":
            self.dv.commit()
        elif s == "reset":
            self.dv.close()
            self.dv.begin()
        elif s == "refresh":
            self.dv.refresh()
        elif s == "table":
            s = self.dv.sql_tables()
            self.s_print(self.single(s))
        elif s == "exit":
            raise Exception("exit")
        elif s.split(" ")[0] == "source":
            # source filepath [encoding]
            arr = s.split(" ")
            fp =arr[1].strip()
            cd = "utf-8"
            if len(arr)>2:
                cd = arr[2].strip().lower()
            with open(fp, 'rb') as f:
                s = f.read().decode(cd)
            arr = s.split(";")
            for sql in arr:
                if sql.strip() == "":
                    continue
                _sql = sql+";"
                self.s_print("sql:", _sql)
                tmp = self.execute(_sql)
                self.s_print(tmp)
            self.s_print("done source", fp)
        elif s.split(" ")[0] == "export":
            # export filepath encoding sql;
            self.s_print("export:", s)
            arr = s.split(" ")
            fp =arr[1].strip()
            cd = arr[2].strip().lower()
            s = " ".join(arr[3:])
            self.s_print("sql:", s)
            rst = self.query(s)
            result = []
            if len(rst)>0:
                keys = rst[0]
                rst = rst[1:]
                result.append(['"'+v.lower()+'"' for v in keys])
                for i, obj in zip(range(len(rst)), rst):
                    v = ['"'+str(k)+'"' for k in obj]
                    result.append(v)
            result = [", ".join(k) for k in result]
            rs = "\n".join(result)
            with open(fp, "wb") as f:
                f.write(rs.encode(cd))
            self.s_print("done write \"{s}\" to {fp}".format(s = s, fp = fp))
        elif s == "":
            return ""
        else:
            self.s_print("SQL:", s)
            try:
                if self.dv.check_query(s):
                    rst = self.query(s)
                    result = []
                    if len(rst)>0:
                        keys = rst[0]
                        rst = rst[1:]
                        result.append(["KEY"]+keys)
                        for i, obj in zip(range(len(rst)), rst):
                            v = [self.jstr(k) for k in obj]
                            result.append([str(i)]+v)
                    self.s_print(self.format(result))
                else:
                    rst = self.execute(s)
                    self.s_print(rst)
            except Exception as exp:
                import traceback
                traceback.print_exc()
                self.s_print("Error sql line:", exp)
            self.s_print("")
        return self.s_flush()
    def run(self):
        self.begin()
        try:
            while True:
                rst = []
                while True:
                    s = input(":").strip()
                    if s in "cls, clear".split(","):
                        import os
                        os.system(s)
                        continue
                    rst.append(s)
                    if s.find(";")>=0:
                        break
                s = " ".join(rst)
                s = s.split(";")[0].strip()
                if s == "exit":
                    break
                rst = self.single(s)
                print(rst)
                print("")
        finally:
            self.close()

pass



#


from .. import xf
from .. import ioc
from ..base import Base
from ..ioc import wrap
from ..tools import *
import time, sys
from ..tz.log import FpLog
@wrap.obj(id="log")
@wrap.obj_sets(cache="ref,cache")
class AutoLog(FpLog):
    def call(self, maps, fp):
        fp = xf.g(maps, log = None)
        fp = self.cache.rfp(fp)
        self.fp = fp
        shows = xf.get(maps, "log.shows")
        if shows is None:
            shows = ["info", "warn", "error"]
        self.shows = shows
        format = xf.get(maps, "log.format")
        if format is not None:
            self.format = format
        #print(f"[TETSZ] format: {format}")
        return True

pass



#coding=utf-8
from buildz import xf, argx, pyz, ioc
from buildz.ioc.ioc.conf import Conf
from ..test import Help as BaseHelp
from os.path import dirname, join
from ..test import text_sfx, args, maps, en
class Help(BaseHelp):
    def __init__(self, dp, fp):
        fp = join(dp, fp)
        self.helps = xf.loads(xf.fread(fp))
        self.text = self.helps['text'+text_sfx]
    def deal(self):
        self._deal()
        if en:
            print("remarks isn't fully translated into English yet.")
    def _deal(self):
        if xf.g(maps, doc=0):
            print(Conf.__doc__)
            return
        fp = argx.get(maps, ('f', 'file'))
        if fp is None:
            fp = ioc.default_deals
        deals = xf.g(xf.loads(xf.fread(fp)), deals=[])
        types = [f"{k['type']}({xf.g(k, note='???')})" for k in deals]
        deals = {k['type']:k for k in deals}
        notes = []
        stypes = ", ".join(types)
        self.text = self.text.replace("{items}",stypes)
        var = argx.get(maps, ('h', 'help'))
        if var == True:
            print(self.text)
            return
        if var not in deals:
            print(f"unknown type: {var}")
            return
        build = deals[var]['build']
        c = pyz.load(build)
        doc = c.__doc__
        if doc is None:
            doc = "没有注释"
        print(doc)
pass



#python
"""
测试代码
测试xf.loads和json.loads的速度
在修改json的scanner.py(不让它用c_make_scanner这个底层优化函数)
耗时对比如下：

json
time cost in <function loads at 0x0000015DEDD0D9E0>: 0.04396796226501465

xf
time cost in <function loads at 0x0000015DEDD0F100>: 0.3307473659515381

慢7倍多，感觉还能接受，之前没注意速度，更早的版本(buildz.xf.read.loads，后面可能会删掉)耗时要三四秒，对比之后就重新写了个，也不管什么堆栈了，就递归调用，后面有时间再改回堆栈吧（python的list的append和pop效率貌似不咋地，尤其是pop(0)）
又写了个C版本的代码来加速，速度大概比C加速版json的慢三到四倍（汗，和没用C加速版的json差不多速度，追求结构化不追求速度，逻辑上本来就多了不少内存分配和运算，用C只是因为C本身比python快，起码满足自己使用了，本测试代码里C加速版xf比python版快7倍，在其他场景里实际使用快了十倍以上，不过也比C版json慢了六七倍），源码暂不公布（C语言要自己写垃圾回收真累，还只是写了个计数器的，还要自己写List和Map，另外就是用C写了后，发现除了当库给其他语言使用，貌似没啥用，还有cython调用C真方便，后续可能会优化下，配置文件格式错误把错误位置打印出来，这个实现起来简单，不过不一定写，不是必要的，还有C版本的垃圾回收在抛错的时候清理下数据，不清理的话，配置文件格式错误会导致之前分配的内存未释放，内存占用增多，因为不打算直接在C用xf，打算抛错的时候，C用的内存全释放算了，之后调用再重新分配）
还有个比较操蛋的点是，用java也写了一版读取xf格式配置文件，结果java版比C版还快，一脸问号，是在C写的内存分配和回收占用时间了吗？更无语的是，java做循环读取配置文件，貌似会触发java的优化，平均速度还会更快（循环100次后和C加速版python的json差不多一样快了），不清楚是因为循环读的是一样的字符串导致的java优化，还是其他原因导致的java优化，如果是其他原因，说明读多个配置文件的时候java版会越来越快，java版会比C版更适合使用，而且java自己有垃圾回收，有List和Map，有时间下个python库用python调java试试
"""
from buildz.xf import readz as rz
from buildz.xf import read as rd
from buildz import xf, fz
import json
import time,sys

try:
    # C加速代码
    import cxf
except:
    from buildz.xf import readz as cxf
    pass
pass
def cost(n, f,*a,**b):
    c = time.time()
    r = f(*a,**b)
    d = time.time()-c
    print(f"time cost in {n}-{f}: {d}")
    return r, d

pass

n = 100
m = 13
l = 12
_arr = [123]
print("test A")
for i in range(n):
    _arr = [list(_arr)]

pass
print("test B")
_map = {}
for i in range(m):
    _map[i] = dict(_map)

pass
print("test C")
rst = []
for i in range(l):
    rst.append([_arr,_map])

pass
print("test D")
json.dumps(_arr)
print("test E")
json.dumps(_map)
print("test F")
js = json.dumps(rst)
#js = fz.read(fp, 'r')
#js = "\n\n"+js+"\n"
#js = xf.dumps(rst, json_format=1)
# js = r"""
# [
#     1,2,3,{"4":5,"6":7,"8":9,"10":11,"4":6}
# ]
# """
print("start")
num = 10
cs = [0,0,0]
for i in range(num):
    jv,cj = cost("json.loads", json.loads,js)
    xv,cx = cost("rz.loads",rz.loads,js)
    cv,cv = cost("cxf.loads", cxf.loads, js)
    cs[0]+=cj
    cs[1]+=cx
    cs[2]+=cv
print(f"judge: {jv==xv}")
print(f"judge: {jv==cv}")
print(f"json mean cost: {cs[0]/num}")
print(f"xf mean cost: {cs[1]/num}")
print(f"cxf mean cost: {cs[2]/num}")
print(f"xf cost =  {'%.3f'%(cs[1]/cs[0],)} json")
print(f"cxf cost = {'%.3f'%(cs[2]/cs[0],)} json")
print(f"xf cost = {'%.3f'%(cs[1]/cs[2],)} cxf")
#_xv = cost("rd.loads",rd.loads, js)
#with open("test.json", 'w') as f:
#    f.write(js)
if n>3 or m>3 or l > 3:
    exit()
print(json.dumps(jv))
print(json.dumps(xv))






import sys
from .structz import ItDv
def sp(obj):
    return super(obj.__class__, obj)

pass
class SimpleDv(ItDv):
    # func to impl:
    def to_list(self, query_result):
        return []
    def sql_tables(self):
        return "select TABLE_NAME tableName,TABLE_COMMENT comemnt,TABLE_ROWS dataNums from information_schema.tables where table_schema='"+self.db+"'"
    def new_con(self):
        return None
    def new_cursor(self):
        return None
    def init(self, *argv, **maps):
        pass
    # func already impl
    def check_query(self, s):
        arr = s.split(" ")
        k = arr[0].strip().lower()
        rst = k not in "delete,insert,update,create,drop,commit".split(",")
        return rst
    def __init__(self, host, port, user, pwd, db, *argv, **maps):
        self.host = host
        self.port = port
        self.user = user
        self.pwd = pwd
        self.db = db
        self.con = None
        self.cursor = None
        self.init(*argv, **maps)
    def begin(self):
        self.con = self.new_con()
        self.cursor = self.new_cursor()
    def close(self):
        self.cursor.close()
        self.con.close()
        self.cursor = None
        self.con = None
    def is_open(self):
        return self.cursor is not None
    def commit(self):
        self.cursor.close()
        self.con.commit()
        self.cursor = self.new_cursor()
    def refresh(self):
        self.cursor.close()
        self.cursor = self.new_cursor()
    def query(self, sql, vals=()):
        # return list, first row is key
        tmp = self.cursor.execute(sql, vals)
        print("[TESTZ] exe:",tmp)
        rst = self.cursor.fetchall()
        return self.to_list(rst)
    def execute(self, sql, vals=()):
        tmp = self.cursor.execute(sql, vals)
        return tmp

pass

# user|pwd@host:port/db
def fetch(args):
    """
        host[:port][/db][ user][ pwd]
        host:port/db user pwd
    """
    if type(args) == str:
        args = args.split(" ")
    url = args[0].strip()
    tmp = url.split("/")
    url = tmp[0]
    db = None
    if len(tmp)>1:
        db = tmp[1]
    tmp = url.split(":")
    if len(tmp)==2:
        host, port = tmp
        port = int(port)
    else:
        host = tmp[0]
        port = None
    user = None
    if len(args)>1:
        user = args[1].strip()
    pwd = None
    if len(args)>2:
        pwd = args[2].strip()
    return host, port, user, pwd, db

pass


#coding=utf-8
from buildz import xf, pyz
from buildz.xf import g as xg
import json
from .base import Base, EncapeData,IOCError, IdNotFoundError
from builtins import id as _id
class Conf(Base):
    """
        配置文件格式：
            {
                _id: 配置文件id，默认null
                //在配置文件配置的环境变量
                envs: {
                    id: val
                    ...
                }
                // 数据配置项处理逻辑，一般不用管
                deals: {
                    {
                        type: 要处理的数据类型
                        build: 函数import的路径
                        args: [] // 列表入参
                        maps: {} // 字典入参
                    }
                }
                id|namespace: 命名空间，默认null
                //本地数据配置项
                locals: [
                    item_conf,
                    ...
                ]
                //全局数据配置项
                datas: [
                    item_conf,
                    ...
                ]
                // 初始化，会一个一个get(id)
                // 其他地方调用该conf.get的时候，会先判断有没有进行init，没有就先调init里的get
                inits: [
                    id,
                    ...
                ]
            }
        如果只有全局数据配置项，可以只写datas里的东西:
            [
                //全局数据配置项
                item_conf,
                ...
            ]
    """
    def get_key(self, obj, key = 'id', index=0):
        if type(obj)==dict:
            return obj[key]
        id = obj[index]
        if type(id) in [list, tuple]:
            return id[0]
        return id
    def map(self, arr, fc_key):
        return {fc_key(obj): obj for obj in arr if fc_key(obj) is not None}
    def __str__(self):
        return f"conf<id={self.namespace}, _id={self.id}>"
    def __repr__(self):
        return self.__str__()
    def init(self, conf, confs):
        """
            {
                deals:[{build: fc_path,args: [],maps: {}}]
                envs: {id: val}
                _id: default null
                id|namespace: default null
                datas: [{id:val, type: val, ...}]
                locals: [like datas]
                default_type: default null
            }
        """
        if type(conf)!=dict:
            conf = {'datas':conf}
        id = xf.g(conf, _id=None)
        if id is None:
            id = confs.conf_id()
        self.id = id
        self.namespace = xf.g1(conf, namespace=None, id=None)
        self.conf = conf
        self.confs = confs
        self.locals = self.map(xf.g(conf, locals=[]), self.confs.get_data_id)
        self.datas = self.map(xf.g(conf, datas=[]), self.confs.get_data_id)
        self.deals = self.map(xf.g(conf, deals = []), self.confs.get_deal_type)
        self.inits = xf.g(conf, inits = [])
        self._default_type = xf.g(conf, default_type = None)
        self.envs = xf.g(conf, envs = {})
        self.confs.flush_env(self.envs)
        self.confs.update_env(self.envs)
        for _type in list(self.deals.keys()):
            conf = self.deals[_type]
            if type(conf) in [list, tuple]:
                maps = {}
                maps['type'] = conf[0]
                maps['build'] = conf[1]
                arr = conf[2:]
                if len(arr)>0:
                    maps['args'] = arr.pop(0)
                if len(arr)>0:
                    maps['maps'] = arr.pop(0)
                conf = maps
            fc = pyz.load(conf["build"])
            args = xf.g(conf, args=[])
            maps = xf.g(conf, maps={})
            deal = fc(*args, **maps)
            self.deals[_type] = deal
            aliases = xf.g(conf, aliases = [])
            for alias in aliases:
                self.deals[alias] = deal
        self.mark_init = False
    def do_init(self):
        if self.mark_init:
            return
        self.mark_init = True
        for id in self.inits:
            self.get(id)
    def get_env(self, id, search_confs = True):
        if self.confs.global_env and search_confs:
            return self.confs.get_env(id, self.id)
        envs = self.confs.get_env_maps(id, self.envs)
        if envs is not None:
            return envs
        if not search_confs:
            return None
        return self.confs.get_env(id, self.id)
    def set_deal(self, type, fc):
        self.deals[type] = fc
        self.confs.set_deal(type, fc)
    def get_deal(self, type, search_confs = True):
        if self.confs.global_deal and search_confs:
            return self.confs.get_deal(type, self.id)
        if type in self.deals:
            return self.deals[type]
        if not search_confs:
            return None
        return self.confs.get_deal(type, self.id)
    def get_data_conf(self, data, src = None, info = None):
        if type(data) not in [list, dict, tuple]:
            i = self.confs.data_index_type[0]
            data = [self.default_type(), data]
            if i != 0:
                data.reverse()
        _type = self.confs.get_data_type(data, 1, self.default_type())
        edata = EncapeData(data, self, local=True, type=_type, src = src, info = info)
        return edata
    def get_data(self, id, local = True, search_confs = True, src = None, info = None):
        self.do_init()
        if type(id) in [list, tuple, dict]:
            return self.get_data_conf(id, src, info)
        if id in self.datas:
            obj = self.datas[id]
            edata = EncapeData(obj, self, local = False, src=src, info = info)
            if _id(obj) != _id(edata.data):
                # 有parent，做了填充，用填充后的替换
                self.datas[id] = edata.data
            return edata
        if not local:
            return None
        if id in self.locals:
            obj = self.locals[id]
            edata = EncapeData(obj, self, local = True, src=src, info = info)
            if _id(obj) != _id(edata.data):
                # 有parent，做了填充，用填充后的替换
                self.locals[id] = edata.data
            return edata
        if not search_confs:
            return None
        gid = self.confs.gid(self.namespace, id)
        obj = self.confs.get_data(gid, self.id, src=src, info = info)
        if obj is not None:
            return obj
        obj = self.confs.get_data(id, self.id, src=src, info = info)
        return obj
    def full_ids(self, local = True):
        rst = []
        arr = [self.datas]
        if local:
            arr.append(self.locals)
        for datas in arr:
            for id in datas:
                gid = self.confs.gid(self.namespace, id)
                item = [gid, id, self]
                rst.append(item)
        return rst
    def get(self, *args, **maps):
        return self.get_obj(*args, **maps)
    def default_type(self):
        if self._default_type is None:
            return self.confs.default_type
        return self._default_type
    def get_var(self, key, i = -1):
        return self.confs.get_var(key, i)
    def var_keys(self):
        return self.confs.var_keys()
    def push_var(self, key, val):
        return self.confs.push_var(key,val)
    def has_var(self, key):
        return self.confs.has_var(key)
    def pop_var(self, key):
        self.confs.pop_var(key)
    def get_obj(self, id, src = None, info=None, remove = False):
        """
            根据data id获取data对象，处理逻辑：根据data id查配置，根据配置的type查deal，返回deal处理过的配置
        """
        self.do_init()
        if type(id) == EncapeData:
            conf = id
        else:
            conf = self.get_data(id, src = src, info = info)
        if conf is None:
            raise IdNotFoundError(f"can't find conf of {id}")
            return None
        if conf.conf is None:
            if remove:
                return None
            return conf.data()
        deal = self.get_deal(conf.type)
        if deal is None:
            raise IOCError(f"can't find deal of {id}, type = {conf.type}")
            return None
        if not remove:
            obj = deal(conf)
        else:
            obj = deal.remove(conf)
        return obj
    def remove(self, id):
        return self.get_obj(id, remove=True)

pass



#coding=utf-8
from buildz.base import Base
from buildz import xf
from threading import Lock
class Decorator(Base):
    def init(self):
        self.conf = {}
        self.confs = {}
        self.confs[None] = self.conf
        self.namespace = None
        self.fcs = {}
        self._ns = {}
        self.regist("add_datas", self.add_datas)
    def fcns(self, namespace, fc):
        self._ns[fc] = namespace
    def ns(self, namespace):
        self.namespace = namespace
    def curr_ns(self):
        return self.namespace
    def regist(self, key, fc):
        self.fcs[key] = fc
    def get_conf(self, src):
        ns = self.namespace
        if src in self._ns:
            ns = self._ns[src]
        if ns not in self.confs:
            conf = {}
            conf['namespace'] = ns
            self.confs[ns] = conf
        return self.confs[ns]
    def get(self, tag, index, src=None):
        conf = self.get_conf(src)
        if tag not in conf:
            conf[tag]=[]
        return conf[tag][index]
    def add(self, tag, data, src = None):
        conf = self.get_conf(src)
        if tag not in conf:
            conf[tag]=[]
        id = len(conf[tag])
        conf[tag].append(data)
        return id
    def set(self, tag, key, val, src=None):
        conf = self.get_conf(src)
        if tag not in conf:
            conf[tag]={}
        conf[tag][key]=val
    def add_datas(self, item, key):
        if type(item)==str:
            item = xf.loads(item)
        return self.add("datas", item, key)
    def get_datas(self, id):
        return self.get("datas", id)
    def set_datas(self, id, val):
        return self.set("datas", id, val)
    def set_envs(self, key, val):
        return self.set("env", key, val)
    def add_inits(self, val):
        return self.add("inits", val)
    def add_locals(self, item):
        return self.add("locals", item)
    def all(self):
        arr = [val for k,val in self.confs.items()]
        return arr
    def call(self):
        return self.conf

pass

decorator = Decorator()
class Fcs:
    pass

pass
class NameSpace(Base):
    def init(self, decorator):
        self.decorator = decorator
        self.lock = Lock()
    def fc(self, namespace, rfc):
        def wfc(*a, **b):
            with self.lock:
                ns = self.decorator.curr_ns()
                self.decorator.ns(namespace)
                rst = rfc(*a,**b)
                self.decorator.fcns(namespace, rst)
                self.decorator.ns(ns)
                return rst
        return wfc
    def call(self, namespace):
        fcs = self.decorator.fcs
        obj = Fcs()
        for k,f in fcs.items():
            setattr(obj, k, self.fc(namespace, f))
        return obj

pass
ns = NameSpace(decorator)


from .. import xf,fz
from .. import ioc
from ..base import Base
from . import init
import os
class Run(Base):
    def init(self, fps=None, init=None, basedir=None):
        mg = ioc.build(init)
        if fps is not None:
            mg.add_fps(fps)
        self.mg = mg
        if basedir is not None:
            self.basedir(basedir)
    def basedir(self, dp):
        self._base = dp
        self.mg.get("cache").set_basedir(dp)
    def add_current(self, dp):
        self.mg.get("cache").add_current(dp)
    def call(self, fp):
        self.log = self.mg.get("log").tag("Run")
        basedir = self.mg.get("cache").get_basedir()
        if basedir is not None and not fz.is_abs(fp):
            fp = os.path.join(basedir, fp)
        if not os.path.isfile(fp):
            dp = os.path.dirname(fp)
            fn = os.path.basename(fp)
            fps = fz.search(dp, fn)
            self.log.info(f"search dp:{dp}, fn: {fn}, rst: {fps}")
            if len(fps)!=1:
                self.log.error(f"can't find filepath: {fp}")
                return False
            fp = fps[0]
        maps = xf.loadf(fp)
        dp = os.path.dirname(os.path.abspath(fp))
        self.add_current(dp)
        config = self.mg.get("buildz.auto.config.load")
        if not config(maps, fp):
            return False
        calls = xf.g(maps, calls = [])
        for deal in calls:
            fc = self.mg.get(deal)
            if not fc(maps, fp):
                return False
        return True

pass




#

from .. import xf, fz
from .. import ioc
from ..base import Base
from ..ioc import wrap
import os,re

@wrap.obj(id="cache.modify")
@wrap.obj_args("ref, cache", "ref, log")
class Update(Base):
    """
    #{}
    """
    def init(self, cache, log, pt = "(#\{([^\{\}]*)\})"):
        self.cache = cache
        self.log = log.tag("Cache.Update")
        self.pt = pt
    def call(self, s):
        if type(s)==dict:
            rst = {}
            for k,v in s.items():
                k,v = self(k), self(v)
                rst[k] = v
            return rst
        elif type(s)==list:
            rst = []
            for v in s:
                v = self(v)
                rst.append(v)
            return rst
        elif type(s)!=str:
            return s
        rst = re.findall(self.pt, s)
        for match, key in rst:
            has = self.cache.has(key)
            val = self.cache.get(key)
            if not has:
                err = f"'{key}' not found in cache"
                self.log.error(err)
                raise Exception(err)
            if s == match:
                s = val
            else:
                s = s.replace(match, str(val))
        return s

pass



@wrap.obj(id="cache.save")
@wrap.obj_args("ref, cache.file", "ref, log")
class Save(Base):
    def init(self, cache, log, fkey = "cache.save", fp = None):
        self.fkey = fkey
        self.cache = cache
        self.log = log.tag("Cache.Save")
        self.fp = fp
    def save(self):
        fp = self.fp
        if fp is None:
            self.log.warn(f"cache not save cause 'cache.save' is None")
            return False
        fz.makefdir(fp)
        rst  = self.cache.data
        rs = xf.dumps(rst, format=True).encode("utf-8")
        fz.write(rs, fp, 'wb')
        return True
    def call(self, maps, fp):
        fp = xf.get(maps, self.fkey, None)
        fp = self.cache.rfp(fp)
        self.fp = fp
        return self.save()

pass
@wrap.obj(id="cache.file")
@wrap.obj_args("mcall, log, tag, [Cache.File]", "env, cache.rfp.current.first, false")
class Cache(Base):
    def has(self, key):
        ks = key.split(".")
        return xf.has(self.data, ks)
    def get(self, key):
        ks = key.split(".")
        return xf.gets(self.data, ks)
    def set(self, key, val):
        xf.sets(self.data, key.split("."), val)
    def remove(self, key):
        xf.removes(self.data, key.split("."))
    def init(self, log, current_first=False, fkey = "cache"):
        self.fkey = fkey
        self.current_first = current_first
        self.log = log
        self.data = {}
    def set_current(self, dp):
        if type(dp)!=list:
            dp = [dp]
        self.set("cache.path.current", dp)
    def add_current(self, dp):
        dps = self.get_current()
        if dps is None:
            dps = []
        if dp in dps:
            return
        dps.append(dp)
        self.set_current(dps)
    def get_current(self):
        dps = self.get("cache.path.current")
        if dps is None:
            dps = []
        if type(dps)!=list:
            dps = [dps]
        return dps
    def set_basedir(self, dp):
        self.set("cache.path.base", dp)
    def get_basedir(self):
        return self.get("cache.path.base")
    def rfp(self, fp):
        if fz.is_abs(fp):
            return fp
        dps = [None,"."]
        cfps = self.get_current()
        if cfps is not None:
            if self.current_first:
                dps = cfps+dps
            else:
                dps = dps+cfps
        basedir = self.get_basedir()
        if basedir is not None:
            dps = [basedir]+dps
        for dp in dps:
            _fp = fp
            if dp is not None:
                _fp = os.path.join(dp, fp)
            if os.path.isfile(_fp):
                return _fp
        basedir = self.get_basedir()
        if basedir is not None:
            fp = os.path.join(basedir, fp)
        return fp
    def call(self, maps, fp):
        fp = xf.get(maps, self.fkey, "cache.js")
        if type(fp)!=list:
            fp = [fp]
        fps=fp
        data = {}
        for fp in fps:
            fp = self.rfp(fp)
            if os.path.isfile(fp):
                self.log.info(f"load cache from {fp}")
                xdata = xf.flush_maps(xf.loadf(fp),visit_list=True)
                xf.fill(xdata, data, replace=1)
        xf.fill(data, self.data, replace=0)
        return True

pass

@wrap.obj(id="cache.mem")
@wrap.obj_args("mcall, log, tag, [Cache.Mem]")
class Mem(Cache):
    def init(self, log, current_first=False, fkey = "mem"):
        super().init(log)

pass
@wrap.obj(id="cache")
@wrap.obj_args("ref, cache.file", "ref, cache.mem")
class Caches(Base):
    def init(self, cache, mem):
        self.cache = cache
        self.mem = mem
        self.caches = [cache, mem]
        self.set = cache.set
        self.remove = cache.remove
        self.call=cache.call
        self.rfp = cache.rfp
        self.get_current = cache.get_current
        self.add_current = cache.add_current
        self.set_current = cache.set_current
        self.set_basedir = cache.set_basedir
        self.get_basedir = cache.get_basedir
        self.has_file = cache.has
        self.has_mem = mem.has
    def get_file(self, key):
        return self.cache.get(key)
    def get_mem(self, key):
        return self.mem.get(key)
    def set_file(self, key, val):
        self.cache.set(key,val)
    def set_mem(self, key, val):
        self.mem.set(key, val)
    def remove_file(self, key):
        self.cache.remove(key)
    def remove_mem(self, key):
        self.mem.remove(key)
    def get(self, key):
        for cache in self.caches:
            v = cache.get(key)
            if v is not None:
                return v
        return None
    def has(self, key):
        for cache in self.caches:
            if cache.has(key):
                return True
        return False

pass



from buildz.html import parse
from buildz.tools import *
fp = "demo.html"
s = fz.read(fp, "rb").decode("utf-8")

obj = parse(s)

# 可以进行多次查询调用，后一次查询在前一次的结果集里做查询
rst = obj.finds("(class,toplist1-tr_1MWDu)").finds("tag=a,target=_blank").data()
rst = obj.searchs("class,toplist1-tr_1MWDu").finds("(tag,a),(tag,a),(target,_blank)").data()
rst = obj.searchs("class,toplist1-tr_1MWDu").searchs("tag,a", "tag, a", "target, _blank").data()

print(f"search rst: {rst}")


#coding=utf-8
#author: Zzz, emails: 1174534295@qq.com, 1309458652@qq.com

# 文件差异
from .myers_diff import update as m_update, myers as m_steps, encode as m_encode, decode as m_decode,count as m_count
from .tio import *
"""
工具模块
toolz简写成tz

"""
__author__ = "Zzz, emails: 1174534295@qq.com, 1309458652@qq.com"



from .. import base


class PrevNextDeal(base.BaseDeal):
    """
        读取下一个字符放缓存里，应放最低优先级
    """
    def prev(self, buffer, queue, pos):
        c = buffer.read(1,1)
        if c is None or len(c)==0:
            return False
        buffer.add(c)
        #pos.update(c)
        return True
    def deal(self, queue, stack):
        if len(queue)==0:
            return False
        obj = queue.pop(0)
        stack.append(obj)
        return True

pass



import os
import sys
dp = os.path.dirname(__file__)
gdp = os.path.join(dp, "lib")

def init_path(dp):
    path = os.environ["PATH"] 
    if sys.platform.lower().find("win")>=0:
        cb = ";"
    else:
        cb = ":"
    path = path +cb+dp
    os.environ["PATH"] = path

pass

#init_path(dp)
import sys
from .basez import SimpleDv, fetch
from .structz import CMD
class Db(SimpleDv):
    # func to impl:
    def to_list(self, rst):
        rows = self.cursor.description
        keys = [k[0].lower() for k in rows]
        result = []
        result.append(keys)
        if rst is None or len(rst)==0:
            return result
        result += rst
        return result
    def new_con(self):
        if self.port is not None:
            add = ":"+str(self.port)
        else:
            add = ""
        try:
            import cx_Oracle as pymysql
        except ModuleNotFoundError:
            raise Exception("module not found, try: pip install cx-Oracle")
        return pymysql.connect(self.user, self.pwd, self.host+add+"/"+self.db)
    def new_cursor(self):
        return self.con.cursor()
    def init(self, *argv, **maps):
        pass

pass
def build(argv, conf):
    k = 'oracle_lib'
    dp = gdp
    if k in conf:
        dp = conf[k]
    init_path(dp)
    #print(f"oracle lib: {dp}")
    dv = Db(*fetch(argv))
    return dv
def buildbk(argv, conf):
    return CMD(make(argv, conf))

pass



#coding=utf-8
from .base import Base, EncapeData,IOCError
from buildz import xf, pyz
import os
dp = os.path.dirname(__file__)
join = os.path.join

class Single(Base):
    """
        virtual deal
        {
            single=1/0
            info: {
                cid: ??
            }
        }
    """
    def init(self, single="single", cid = "id", default=1):
        self.k_cid = cid
        self.k_single = single
        self.default = default
        self.singles = {}
    def get_ids(self, edata: EncapeData):
        sid = edata.sid
        data = edata.data
        info = edata.info
        if type(info) == dict:
            cid = xf.get(info, self.k_cid, None)
        else:
            cid = None
        id = xf.g(data, id = None)
        single = xf.get(data, self.k_single, None)
        if id is None:
            single = 0
        if single is None:
            single = self.default
        ids = None
        if single or cid is not None:
            if cid is not None:
                ids = [sid, 'local_id', id, cid]
            else:
                ids = [sid, 'single', id]
        return ids
    def get_by_ids(self, ids):
        if ids is None:
            return None
        obj = xf.gets(self.singles, ids)
        return obj
    def get(self, edata:EncapeData):
        ids = self.get_ids(edata)
        return self.get_by_ids(ids)
    def set_by_ids(self, ids, obj):
        if ids is None:
            return
        xf.sets(self.singles, ids, obj)
    def set(self, edata:EncapeData, obj):
        if not isinstance(edata, EncapeData) and isinstance(obj, EncapeData):
            edata, obj = obj, edata
        ids = self.get_ids(edata)
        self.set_by_ids(ids, obj)
    def rm_by_ids(self, idsj):
        if ids is None:
            return
        xf.removes(self.singles, ids)
    def remove(self,edata:EncapeData):
        ids = self.get_ids(edata)
        self.rm_by_ids(ids)

pass



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: Zzz(1309458652@qq.com)
# Description:

from setuptools import setup, find_packages

setup(
    name = 'buildz',
    version = '0.6.28',
    keywords='buildz',
    long_description=open('README.md', 'r', encoding="utf-8").read(),
    long_description_content_type='text/markdown',
    description = "配置读写（基于json格式进行简化）、ioc、以及其他工具代码。a json-base file format's read and write code by python, and codes to read and product object from configure file in such format(ioc), and other tool codes",
    license = 'Apache License 2.0',
    url = 'https://github.com/buildCodeZ/buildz',
    author = 'Zzz',
    author_email = '1309458652@qq.com',
    packages = find_packages(),
    include_package_data = True,
    platforms = 'any',
    install_requires = [],
)



from .. import base
from .. import item
from .. import exp
from ... import file
from ... import code as codez
from . import lr

def init():
    cs = "abfnrtv\\'\"?0123456789xcde"
    maps = {k:k.encode()[0] for k in cs}
    global c2b
    c2b = maps
    global special_keys
    special_keys = [-1]*256
    s  = "abfnrtv\\'\""
    ts = b"\a\b\f\n\r\t\v\\\'\""
    for c,t in zip(s, ts):
        special_keys[c2b[c]] = t
    symbal_a = [False]*256
    for c in s:
        symbal_a[c2b[c]] = True
    global id_0,id_a,id_x,id_A
    global c2nums
    c2nums = [-1]*256
    id_0,id_a,id_A,id_x,id_u = b'0aAxu'
    for i in range(128):
        v = i-id_0
        if v>=0 and v<=9:
            c2nums[i] = v
            continue
        v = i-id_a
        if v>=0 and v<=5:
            c2nums[i]=v+10
            continue
        v = i-id_A
        if v>=0 and v<=5:
            c2nums[i]=v+10

pass
init()
def cs2num(bts, base, min, max, oct = False):
    cnt = 0
    rst = 0
    mv = 4
    if oct:
        mv = 3
    for i in range(max):
        if base+i>=len(bts):
            if i<min:
                return -1,0
            else:
                break
        bt = bts[base+i]
        v = c2nums[bt]
        if v<0 or (oct and v>9):
            if i<min:
                return -1,0
            else:
                break
        cnt+=1
        rst = (rst<<mv)|v
    return rst, cnt

pass
def decode_u(val):
    """
    1 byte: 7 0? 
    2 byte: 11 110? 10?
    3 byte: 16 1110? 10? 10?
    4 byte: 21 11110? 10? 10? 10?
    """
    if val<0x80:
        return [val]
    elif val<0x800:
        b0 = 0x80|(val&0x7f)
        b1 = 0x60|(val>>6)
        return [b1, b0]
    elif val<0x10000:
        b0 = 0x80|(val&0x3f)
        val>>=6
        b1 = 0x80|(val&0x3f)
        b2 = 0xe0|(val>>6)
        return [b2,b1,b0]
    else:
        b0 = 0x80|(val&0x3f)
        val>>=6
        b1 = 0x80|(val&0x3f)
        val>>=6
        b2 = 0x80|(val&0x3f)
        b3 = 0xf0|(val>>6)
        return [b3,b2,b1,b0]

pass
def translate_bts(bts, octs = None, hexs = None):
    i = 0
    rs = []
    while i<len(bts):
        c = bts[i]
        i+=1
        if c!=c2b['\\']:
            rs.append(c)
            continue
        x0 = bts[i]
        i+=1
        if special_keys[x0]>=0:
            rs.append(special_keys[x0])
            continue
        if x0==b'u'[0]:
            c_val, c_cnt = cs2num(bts, i, 4,4, 0)
            if c_cnt==0:
                raise Exception("\\uXXXX error")
            tmp = decode_u(c_val)
            i+=c_cnt
            rs+=tmp
            continue
        if x0==id_x:
            c_val, c_cnt = cs2num(bts, i, 2, 2, 0)
            if c_cnt==0:
                raise Exception("\\xXX error")
            if hexs is not None:
                tmp = hexs[c_val]
            else:
                tmp = [c_val]
            rs+=tmp
            i+=c_cnt
            continue
        c_val,c_cnt = cs2num(bts, i-1, 1, 3, 1)
        if c_cnt>0:
            if octs is not None:
                tmp = octs[c_val]
            else:
                tmp = [c_val%256]
            rs+=tmp
            i+=c_cnt-1
            continue
        rs.append(c)
        rs.append(x0)
        #i-=1
    return bytes(rs)

pass
def gen_chars(code="utf-8"):
    simple = "abfnrtv\\'\""
    octs = [0]*512
    hexs = [0]*256
    for i in range(512):
        if i<256:
            vhex = hex(i)[2:]
            if len(vhex)==1:
                vhex = "0"+vhex
            cmd = f"'\\x{vhex}'"
            hexs[i] = list(eval(cmd).encode(code))
        voct = oct(i)[2:]
        cmd = f"'\\{voct}'"
        octs[i] = list(eval(cmd).encode(code))
    return octs, hexs

pass
class PrevStrDeal(lr.LRDeal):
    def types(self):
        if not self.deal_build:
            return []
        return ['str']
    def build(self, obj):
        obj.is_val = 1
        if self.translate:
            val = obj.val
            val = self.do_translate(val)
            obj.val = val
        return obj
    def prepare(self, mg):
        super().prepare(mg)
        self.as_bytes = mg.as_bytes
        if not self.as_bytes:
            self.octs,self.hexs = gen_chars()
        self.label_l2 = mg.like("\\")
        self.label_qt = mg.like('"')
        self.label_et = mg.like("\n")
        self.label_lr = mg.like("\r")
        self.label_nl = mg.like("")
        self.et_in_right = self.right.count(self.label_et)
    def init(self, left = '"', right= '"', single_line = False, note = False, translate = False, deal_build = False):
        super().init(left, right, 'str')
        self.single_line = single_line
        self.note = note
        self.translate = translate
        self.deal_build = deal_build
        self.as_bytes = True
        self.octs = None
        self.hexs = None
    def json_loads(self, s):
        import json
        x = s
        cd = None
        if type(x)==bytes:
            x, cd = file.decode_c(x)
        rs = json.loads(x)
        if type(s)==bytes:
            rs = rs.encode(cd)
        return rs
    def do_translate(self, s):
        """
            取巧用python的eval来生成字符表
        """
        is_bytes = type(s)==bytes
        if is_bytes:
            return codez.ubytes(s, "utf-8")
        else:
            return codez.ub2s(s.encode("utf-8"), "utf-8")
            return codez.ustr(s)
        if not is_bytes:
            s = s.encode("utf-8")
        #s = s.decode("unicode_escape")
        s = translate_bts(s, self.octs, self.hexs)
        if is_bytes:
            #s = s.encode("utf-8")
            s = s.decode("utf-8")
        return s
        """
            取巧直接调用json
        """
        qt = self.label_qt
        ql = self.label_l2
        et = self.label_et
        tr = self.label_lr
        nt = self.label_nl
        pt = ql+qt
        arr = s.split(pt)
        arr = [k.replace(qt, pt) for k in arr]
        s = pt.join(arr)
        #s = s.replace(qt, ql+qt)
        s = s.replace(tr, nt)
        arr = s.split(et)
        outs = [self.json_loads(qt+k+qt) for k in arr]
        outs = et.join(outs)
        return outs
    def deal(self, buffer, rst, mg):
        cl = buffer.read(self.ll)
        if cl != self.left:
            return False
        rm = buffer.full().strip()
        buffer.clean2read(self.ll)
        if len(rm)>0:
            if not self.note:
                raise Exception(f"unexcept char before string: {rm}")
            else:
                rst.append(item.Item(rm, type = "str", is_val = 0))
        tmp = cl[:0]
        ctmp = tmp[:0]
        do_judge = 1
        mark_et = 0
        mark_l2 = 0
        while True:
            if do_judge and self.right == buffer.rget(self.lr):
                break
            c = buffer.read_cache(1)
            if do_judge and c == self.label_et:
                mark_et += 1
            if len(c)==0:
                if self.single_line and self.note:
                    break
                raise Exception(f"unexcept string end while reading str")
            do_judge = 1
            if c == self.label_l2:
                mark_l2 = 1
                do_judge = 0
                c = buffer.read_cache(1)
                if len(c)==0:
                    raise Exception(f"unexcept string end while reading str")
        data = buffer.full()
        data = data[:-self.lr]
        buffer.clean()
        mark_et -= self.et_in_right
        if self.single_line and mark_et>0:
            print("left:",self.left, "right:", self.right)
            raise Exception(f"contain enter in single line string")
        if self.translate and mark_l2:
            data = self.do_translate(data)
        if self.note:
            return True
        rst.append(item.Item(data, type='str', is_val = 1))
        return True

pass




from . import itemz
class Manager:
    def __init__(self):
        self.deals = []
    def add(self, fc):
        self.deals.append(fc)
    def deal(self, obj, conf):
        if obj.check(is_done=1):
            return obj
        for fc in self.deals:
            rst = fc(obj, conf)
            if rst is not None:
                return rst
        raise Exception("undealable type"+str(obj))
    def dump(self, obj, conf):
        stack_src = [[obj]]
        stack_build = [['root']]
        while len(stack_src)>0:
            crr = stack_src[-1]
            done = 1
            while len(crr)>0:
                obj = crr.pop(0)
                tobj = type(obj)
                if tobj == dict:
                    ncrr = []
                    for k in obj:
                        v = obj[k]
                        ncrr+=[k,v]
                    stack_src.append(ncrr)
                    stack_build.append(['dict'])
                    done = 0
                    break
                elif tobj in [list, tuple]:
                    ncrr = list(obj)
                    stack_src.append(ncrr)
                    stack_build.append(['list'])
                    done = 0
                    break
                else:
                    stack_build[-1].append(itemz.ShowItem(obj, is_val=1))
            if done:
                stack_src.pop(-1)
                items = stack_build.pop(-1)
                rst = []
                for v in items[1:]:
                    v = self.deal(v, conf)
                    rst.append(v)
                if items[0] != 'root':
                    if items[0] == 'list':
                        obj = itemz.ShowItem(rst, is_list=1)
                    else:
                        obj = itemz.ShowItem(rst, is_dict=1)
                    obj = self.deal(obj, conf)
                    stack_build[-1].append(obj)
                else:
                    stack_build.append(rst[0])
        return stack_build[0].val

pass